@@ -19,6 +19,8 @@
  * 		Todd Kulesza <todd@dropline.net>
  */
 
+#include <config.h>
+
 #include <string.h>
 #include <glib.h>
 #include <curl/curl.h>
@@ -27,20 +29,67 @@
 #include "drivel.h"
 #include "journal.h"
 #include "dialogs.h"
+#include "login.h"
 #include "network.h"
 #include "tray.h"
 #include "utils.h"
 #include "blog_lj.h"
 
+static void
+parse_friendgroup_data (DrivelRequest *dr, GtkListStore *store)
+{
+	const gchar *mesg;
+	
+	/* fetch the user's security groups */
+	mesg = drivel_request_value_lookup (dr, "frgrp_maxnum");
+	if (mesg)
+	{
+		gchar group_sort[32], group_name[32], group_public[32];
+		const gchar *name, *sort, *public;
+		gboolean is_public;
+		gint i, groups;
+		
+		groups = (gint)g_strtod (mesg, NULL);
+		
+		for (i = 1; i <= groups; i++)
+		{
+			g_snprintf (group_name, 32, "frgrp_%d_name", i);
+			g_snprintf (group_sort, 32, "frgrp_%d_sortorder", i);
+			g_snprintf (group_public, 32, "frgrp_%d_public", i);
+			
+			name = drivel_request_value_lookup (dr, group_name);
+			sort = drivel_request_value_lookup (dr, group_sort);
+			public = drivel_request_value_lookup (dr, group_public);
+			
+			if (public)
+				is_public = (gboolean)g_strtod (public, NULL);
+			else
+				is_public = FALSE;
+			
+			if (name && sort)
+			{
+				gint sortorder;
+				
+				sortorder = (gint)g_strtod (sort, NULL);
+				store_security_append (store, "protected.png", name, i, 
+						sortorder, is_public);
+			}
+		}
+	}
+	
+	return;
+}
+
 DrivelRequest*
-blog_lj_build_login_request (const gchar *username, const gchar *version, 
-		gint n_moods)
+blog_lj_build_login_request (const gchar *username, const gchar *uri, 
+		const gchar *version, gint n_moods)
 {
 	DrivelRequest *dr;
 	
 	dr = drivel_request_new_with_items (REQUEST_TYPE_LOGIN, 
 			REQUEST_PROTOCOL_POST, 
 			BLOG_API_LJ,
+			uri,
 			g_strdup ("mode"), g_strdup ("login"),
 			g_strdup ("user"), curl_escape (username, 0),
 			g_strdup ("ver"), g_strdup ("1"),
@@ -58,48 +107,18 @@ void
 blog_lj_parse_login_request (DrivelClient *dc, DrivelRequest *dr)
 {
 	const gchar *mesg, *key, *value;
-	gchar *login, *password, static_key[64];
-	gboolean save_password, autologin;
-	GSList *users;
+	gchar static_key[64];
 	gint pictures, i, j, offset, menus;
 	GtkWidget *dialog;
+	DrivelRequest *dr2;
 	LJMenuItem *menu_item;
+	DrivelJournal *dj;
 	
-	login = gtk_editable_get_chars (GTK_EDITABLE (GTK_BIN (dc->login_name)->child), 0, -1);
-	password = gtk_editable_get_chars (GTK_EDITABLE (dc->login_password), 0, -1);
-	save_password = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dc->save_password));
-	autologin = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dc->autologin));
+	add_account_to_list (dc);
 	
-	if (!save_password)
-	{
-		g_free (password);
-		password = g_strdup ("");
-	}
+	/* clear out the list of security groups and add the defaults */
+	fill_security_menu (dc, dc->security_store);
 	
-	users = gconf_client_get_list (dc->client, "/apps/drivel/global_settings/user_list", GCONF_VALUE_STRING, NULL);
-	
-	/* if this is the first time this user has logged in, add them to the user 
-	 * list and set their default preferences. */
-	if (users != NULL && g_slist_find_custom (users, login, string_compare))
-	{
-	}
-	else
-	{
-		users = g_slist_prepend (users, login);
-		gconf_client_set_list (dc->client, "/apps/drivel/global_settings/user_list", GCONF_VALUE_STRING, users, NULL);
-		gconf_client_set_bool (dc->client, dc->gconf->tray, TRUE, NULL);
-	}
-	
-	drivel_gconf_data_fill (dc->gconf, login, dc->client, &dc->id, dc);
-	
-	gconf_client_set_string (dc->client, "/apps/drivel/global_settings/current_user", login, NULL);
-	gconf_client_set_string (dc->client, dc->gconf->password, password, NULL);
-	gconf_client_set_bool (dc->client, dc->gconf->save_password, save_password, NULL);
-	gconf_client_set_bool (dc->client, dc->gconf->autologin, autologin, NULL);
-
-	g_free (password);
-	g_free (login);
-
 	mesg = drivel_request_value_lookup (dr, "success");
 	if (!mesg || !strcmp (mesg, "FAIL"))
 	{
@@ -107,10 +126,7 @@ blog_lj_parse_login_request (DrivelClient *dc, DrivelRequest *dr)
 		display_error_dialog (dc, _("Server error"), mesg);
 		return;
 	}
-	
-	/* put our login name into the list of journals we can post to */
-	dc->journal_list = g_slist_append (dc->journal_list, g_strdup (dc->username));
-	
+
 	/* if lj sent us a message, display it to the user */
 	mesg = drivel_request_value_lookup (dr, "message");
 	if (mesg)
@@ -127,13 +143,6 @@ blog_lj_parse_login_request (DrivelClient *dc, DrivelRequest *dr)
 		gtk_widget_destroy (dialog);
 	}
 	
-	/* get the user's lj name */
-	mesg = drivel_request_value_lookup (dr, "name");
-	if (mesg)
-		dc->name = g_strdup (mesg);
-	else
-		dc->name = g_strdup ("unknown");
-	
 	/* are we able to use the fast servers? */
 	mesg = drivel_request_value_lookup (dr, "fastserver");
 	if (mesg && g_strtod (mesg, NULL))
@@ -269,6 +278,9 @@ blog_lj_parse_login_request (DrivelClient *dc, DrivelRequest *dr)
 		gconf_client_set_list (dc->client, "/apps/drivel/moods/mood_list", GCONF_VALUE_STRING, dc->mood_list, NULL);
 	}
 	
+	/* fetch the user's security groups */
+	parse_friendgroup_data (dr, dc->security_store);
+	
 	/* # of journals this user has access to */
 	mesg = drivel_request_value_lookup (dr, "access_count");
 	if (mesg)
@@ -279,15 +291,38 @@ blog_lj_parse_login_request (DrivelClient *dc, DrivelRequest *dr)
 		{
 			g_snprintf (static_key, 64, "access_%d", i + 1);
 			mesg = drivel_request_value_lookup (dr, static_key);
-			if (strncmp (mesg, dc->username, strlen (mesg)))
-				dc->journal_list = g_slist_append (dc->journal_list, g_strdup (mesg));
+			if (strncmp (mesg, dc->user->username, strlen (mesg)))
+			{
+				dj = drivel_journal_new ();
+				dj->name = g_strdup (mesg);
+				dj->uri_view = g_strdup_printf ("%s/users/%s",
+						dc->user->server, mesg);
+				dj->type = JOURNAL_TYPE_COMMUNITY;
+				dc->journal_list = g_slist_prepend (dc->journal_list, dj);
+			}
 		}
 	}
 	else
 		dc->journals = 0;
-	/* add the user's main journal to the list */
+	
+	/* put our login name into the list of journals we can post to */
+	dj = drivel_journal_new ();
+	dj->name = g_strdup (dc->user->username);
+	dj->uri_view = g_strdup_printf ("%s/users/%s",
+						dc->user->server, dj->name);
+	/* get the user's journal name */
+	mesg = drivel_request_value_lookup (dr, "name");
+	if (mesg)
+		dj->description = g_strdup (mesg);
+	dc->journal_list = g_slist_prepend (dc->journal_list, dj);
+	dc->journal_list = g_slist_sort (dc->journal_list, (GCompareFunc)sort_journals);
 	dc->journals++;
-	dc->journal_list = g_slist_append (dc->journal_list, g_strdup (dc->username));
+
+	/* fetch recent entries for our journal */
+	dr2 = blog_lj_build_getevents_request (dc->user->username, 
+			dc->user->server, 0, FALSE, FALSE, "lastn", NULL, 0, 0, 0, 
+			DRIVEL_N_RECENT_POSTS, NULL, 0, dj);
+	net_enqueue_request (dc, dr2);
 	
 	/* create the web links menu */
 	for (i = 0; ; i++)
@@ -339,7 +374,7 @@ blog_lj_build_getpicture_request (const gchar *url, const gchar *filename)
 	dr = drivel_request_new_with_items (REQUEST_TYPE_GETPICTURE, 
 			REQUEST_PROTOCOL_GET, 
 			BLOG_API_LJ,
-			g_strdup ("url"), g_strdup (url),
+			url,
 			g_strdup ("filename"), g_strdup (filename),
 			NULL);
 	
@@ -390,7 +425,7 @@ blog_lj_parse_getpicture_request (DrivelClient *dc, DrivelRequest *dr)
 }
 
 DrivelRequest*
-blog_lj_build_checkfriends_request (const gchar *username, 
+blog_lj_build_checkfriends_request (const gchar *username, const gchar *uri,
 		const gchar* lastupdate)
 {
 	DrivelRequest *dr;
@@ -398,6 +433,7 @@ blog_lj_build_checkfriends_request (const gchar *username,
 	dr = drivel_request_new_with_items (REQUEST_TYPE_CHECKFRIENDS, 
 			REQUEST_PROTOCOL_POST, 
 			BLOG_API_LJ,
+			uri,
 			g_strdup ("mode"), g_strdup ("checkfriends"),
 			g_strdup ("user"), curl_escape (username, 0),
 			g_strdup ("ver"), g_strdup ("1"),
@@ -431,7 +467,7 @@ blog_lj_parse_checkfriends_request (DrivelClient *dc, DrivelRequest *dr)
 		if ((gint) g_strtod (mesg, NULL))
 		{
 			dc->friends_update = TRUE;
-			tray_icon_update (TRAY_UPDATED_FRIENDS);
+			tray_display (TRUE);
 		}
 		else
 			dc->friends_update = FALSE;
@@ -445,18 +481,25 @@ blog_lj_parse_checkfriends_request (DrivelClient *dc, DrivelRequest *dr)
 }
 
 DrivelRequest*
-blog_lj_build_postevent_request (const gchar *username,
+blog_lj_build_postevent_request (const gchar *username, const gchar *uri,
 		const gchar *event, const gchar *music, const gchar *mood,
 		gint moodid, const gchar *subject, const gchar *security, gint mask,
 		const gchar *picture, gint year, gint month, gint day, gint hour,
-		gint minute, gint nocomments, gint preformatted, const gchar *journal)
+		gint minute, gint nocomments, gint preformatted, const DrivelJournal *dj)
 {
 	DrivelRequest *dr;
+	const gchar *journal;
 	gchar text[8];
 	
+	if (dj && dj->type == JOURNAL_TYPE_COMMUNITY)
+		journal = dj->name;
+	else
+		journal = NULL;
+		
 	dr = drivel_request_new_with_items (REQUEST_TYPE_POSTEVENT, 
 			REQUEST_PROTOCOL_POST, 
 			BLOG_API_LJ,
+			uri,
 			g_strdup ("mode"), g_strdup ("postevent"),
 			g_strdup ("user"), curl_escape (username, 0),
 			g_strdup ("ver"), g_strdup ("1"),
@@ -516,18 +559,26 @@ blog_lj_build_postevent_request (const gchar *username,
 }
 
 DrivelRequest*
-blog_lj_build_editevent_request (const gchar *username, const gchar *itemid,
-		const gchar *event, const gchar *music, const gchar *mood,
-		gint moodid, const gchar *subject, const gchar *security, gint mask,
-		const gchar *picture, gint year, gint month, gint day, gboolean newdate,
-		gint nocomments, gint preformatted, const gchar *journal)
+blog_lj_build_editevent_request (const gchar *username, const gchar *uri,
+		const gchar *itemid, const gchar *event, const gchar *music, 
+		const gchar *mood, gint moodid, const gchar *subject, 
+		const gchar *security, gint mask, const gchar *picture, gint year, 
+		gint month, gint day, gboolean newdate, gint nocomments, 
+		gint preformatted, const DrivelJournal *dj)
 {
 	DrivelRequest *dr;
+	const gchar *journal;
 	gchar text[8];
 	
+	if (dj && dj->type == JOURNAL_TYPE_COMMUNITY)
+		journal = dj->name;
+	else
+		journal = NULL;
+	
 	dr = drivel_request_new_with_items (REQUEST_TYPE_EDITEVENT, 
 			REQUEST_PROTOCOL_POST, 
 			BLOG_API_LJ,
+			uri,
 			g_strdup ("mode"), g_strdup ("editevent"),
 			g_strdup ("user"), curl_escape (username, 0),
 			g_strdup ("ver"), g_strdup ("1"),
@@ -595,28 +646,32 @@ blog_lj_parse_postevent_request (DrivelClient *dc, DrivelRequest *dr)
 		return;
 	}
 	
-	journal_edit_entry_finished (dc);
-	
-	remove_autosave (dc);
-	
-	if (gconf_client_get_bool (dc->client, dc->gconf->min_post, NULL))
-		gtk_window_iconify (GTK_WINDOW (dc->journal_window));
+	net_ping_technorati (dc);
+	journal_finished_post (dc);
 	
 	return;
 }
 
 DrivelRequest*
-blog_lj_build_getevents_request (const gchar *username, gint truncate,
-		gboolean prefersubject, gboolean noprops, const gchar *selecttype,
-		const gchar *lastsync, gint year, gint month, gint day, gint howmany, 
-		const gchar *beforedate, gint itemid, const gchar *journal)
+blog_lj_build_getevents_request (const gchar *username, const gchar *uri,
+		gint truncate, gboolean prefersubject, gboolean noprops, 
+		const gchar *selecttype, const gchar *lastsync, gint year, gint month, 
+		gint day, gint howmany, const gchar *beforedate, gint itemid, 
+		const DrivelJournal *dj)
 {
 	DrivelRequest *dr;
+	const gchar *journal;
 	gchar text[8];
 	
+	if (dj && dj->type == JOURNAL_TYPE_COMMUNITY)
+		journal = dj->name;
+	else
+		journal = NULL;
+	
 	dr = drivel_request_new_with_items (REQUEST_TYPE_GETEVENTS, 
 			REQUEST_PROTOCOL_POST, 
 			BLOG_API_LJ,
+			uri,
 			g_strdup ("mode"), g_strdup ("getevents"),
 			g_strdup ("user"), curl_escape (username, 0),
 			g_strdup ("ver"), g_strdup ("1"),
@@ -675,6 +730,12 @@ blog_lj_build_getevents_request (const gchar *username, gint truncate,
 				g_strdup ("usejournal"), curl_escape (journal, 0), NULL);
 	}
 	
+	if (!strcmp (selecttype, "lastn"))
+	{
+		drivel_request_add_items (dr,
+				g_strdup ("lastn"), g_strdup ("true"), NULL);
+	}
+	
 	return dr;
 }
 
@@ -778,15 +839,74 @@ blog_lj_parse_getevents_request (DrivelClient *dc, DrivelRequest *dr)
 		}
 	}
 	
-	if (dc->edit_entry)
+	if (drivel_request_item_lookup (dr, "lastn"))
+	{
+		for (i = 0; i < events_count; i++)
+		{
+			DrivelJournalProp *prop ;
+			DrivelJournalEntry *entry = journal_entry_new ();
+			
+			entry->content = g_strdup (new_event[i]);
+			if (subject[i])
+				entry->subject = g_strdup (subject[i]);
+			entry->userid = g_strdup (drivel_request_item_lookup (dr, "user"));
+			entry->postid = g_strdup (itemid[i]);
+			entry->security = g_strdup (security[i]);
+			entry->security_mask = g_strdup (allowmask[i]);
+			if (picture[i])
+			{
+				prop = journal_prop_new ();
+				prop->name = g_strdup ("picture");
+				prop->value = g_strdup (picture[i]);
+				g_array_append_val (entry->properties, prop);
+			}
+			if (music[i])
+			{
+				prop = journal_prop_new ();
+				prop->name = g_strdup ("music");
+				prop->value = g_strdup (picture[i]);
+				g_array_append_val (entry->properties, prop);
+			}
+			if (mood[i])
+			{
+				prop = journal_prop_new ();
+				prop->name = g_strdup ("mood");
+				prop->value = g_strdup (picture[i]);
+				g_array_append_val (entry->properties, prop);
+			}
+			if (comments[i])
+			{
+				prop = journal_prop_new ();
+				prop->name = g_strdup ("comments");
+				prop->value = g_strdup (picture[i]);
+				g_array_append_val (entry->properties, prop);
+			}
+			if (autoformat[i])
+			{
+				prop = journal_prop_new ();
+				prop->name = g_strdup ("autoformat");
+				prop->value = g_strdup (picture[i]);
+				g_array_append_val (entry->properties, prop);
+			}
+			prop = journal_prop_new ();
+			prop->name = g_strdup ("eventtime");
+			prop->value = g_strdup (eventtime[i]);
+			g_array_append_val (entry->properties, prop);
+			
+			g_array_append_val (dc->recent_entries, entry);
+		}
+		
+		journal_refresh_recent_entries (dc);
+	}
+	else if (dc->edit_entry)
 	{	
 		journal_edit_entry (dc, itemid [0], new_event [0], security [0], 
 				allowmask [0], subject [0],	mood [0], music [0], picture [0], 
-				eventtime [0], comments [0], autoformat [0]);
+				eventtime [0], comments [0], autoformat [0], NULL);
 	}
 	else
 		update_history_list (dc, itemid, new_event, eventtime, events_count);
-
+	
 	g_strfreev (itemid);
 	g_strfreev (new_event);
 	g_strfreev (old_event);
@@ -804,13 +924,21 @@ blog_lj_parse_getevents_request (DrivelClient *dc, DrivelRequest *dr)
 }
 
 DrivelRequest*
-blog_lj_build_getdaycounts_request (const gchar *username, const gchar *journal)
+blog_lj_build_getdaycounts_request (const gchar *username, const gchar *uri,
+		const DrivelJournal *dj)
 {
 	DrivelRequest *dr;
-
+	const gchar *journal;
+	
+	if (dj && dj->type == JOURNAL_TYPE_COMMUNITY)
+		journal = dj->name;
+	else
+		journal = NULL;
+	
 	dr = drivel_request_new_with_items (REQUEST_TYPE_GETDAYCOUNTS, 
 			REQUEST_PROTOCOL_POST, 
 			BLOG_API_LJ,
+			uri,
 			g_strdup ("mode"), g_strdup ("getdaycounts"),
 			g_strdup ("user"), curl_escape (username, 0),
 			g_strdup ("ver"), g_strdup ("1"),
@@ -863,8 +991,9 @@ blog_lj_parse_getdaycounts_request (DrivelClient *dc, DrivelRequest *dr)
 }
 
 DrivelRequest*
-blog_lj_build_editfriends_request (const gchar *username, const gchar *friend,
-		gboolean delete, gboolean new, const gchar *fg, const gchar *bg)
+blog_lj_build_editfriends_request (const gchar *username, const gchar *uri,
+		const gchar *friend, gboolean delete, gboolean new, const gchar *fg, 
+		const gchar *bg)
 {
 	DrivelRequest *dr;
 	gchar *text;
@@ -872,6 +1001,7 @@ blog_lj_build_editfriends_request (const gchar *username, const gchar *friend,
 	dr = drivel_request_new_with_items (REQUEST_TYPE_EDITFRIENDS, 
 			REQUEST_PROTOCOL_POST, 
 			BLOG_API_LJ,
+			uri,
 			g_strdup ("mode"), g_strdup ("editfriends"),
 			g_strdup ("user"), curl_escape (username, 0),
 			g_strdup ("ver"), g_strdup ("1"),
@@ -912,7 +1042,7 @@ blog_lj_build_editfriends_request (const gchar *username, const gchar *friend,
 void
 blog_lj_parse_editfriends_request (DrivelClient *dc, DrivelRequest *dr)
 {
-	const gchar *mesg;
+	const gchar *mesg, *uri;
 	DrivelRequest *dr_new;
 	
 	mesg = drivel_request_value_lookup (dr, "success");
@@ -923,24 +1053,34 @@ blog_lj_parse_editfriends_request (DrivelClient *dc, DrivelRequest *dr)
 		return;
 	}
 	
-	dr_new = blog_lj_build_getfriends_request (dc->username, TRUE);
+	uri = drivel_request_get_uri (dr);
+	dr_new = blog_lj_build_getfriends_request (dc->user->username, uri, TRUE, FALSE);
 	net_enqueue_request (dc, dr_new);
 
 	return;
 }
 
 DrivelRequest*
-blog_lj_build_getfriends_request (const gchar *username, gboolean friendof)
+blog_lj_build_getfriends_request (const gchar *username, const gchar *uri, 
+		gboolean friendof, gboolean groups)
 {
 	DrivelRequest *dr;
-
-	dr = drivel_request_new_with_items (REQUEST_TYPE_GETFRIENDS, 
+	DrivelRequestType type;
+	
+	if (groups)
+		type = REQUEST_TYPE_GETFRIENDGROUPS;
+	else
+		type = REQUEST_TYPE_GETFRIENDS;
+	
+	dr = drivel_request_new_with_items (type, 
 			REQUEST_PROTOCOL_POST, 
 			BLOG_API_LJ,
+			uri,
 			g_strdup ("mode"), g_strdup ("getfriends"),
 			g_strdup ("user"), curl_escape (username, 0),
 			g_strdup ("ver"), g_strdup ("1"),
 			g_strdup ("includefriendof"), g_strdup_printf ("%d", friendof),
+			g_strdup ("includegroups"), g_strdup_printf ("%d", groups),
 			NULL);
 	
 	return dr;
@@ -989,7 +1129,7 @@ blog_lj_parse_getfriends_request (DrivelClient *dc, DrivelRequest *dr)
 		if (mesg)
 			friend->groupmask = (gint) g_strtod (mesg, NULL);
 		else
-			friend->groupmask = 1;
+			friend->groupmask = 0;
 		g_snprintf (key, 32, "friend_%d_type", i);
 		mesg = drivel_request_value_lookup (dr, key);
 		if (!mesg)
@@ -1044,7 +1184,16 @@ blog_lj_parse_getfriends_request (DrivelClient *dc, DrivelRequest *dr)
 		}
 	}
 	
-	display_edit_friends_dialog (dc);
+	mesg = drivel_request_item_lookup (dr, "includegroups");
+	if (mesg && (gboolean)g_strtod (mesg, NULL))
+	{
+		/* fetch the user's security groups */
+		gtk_list_store_clear (dc->security_store_filtered);
+		parse_friendgroup_data (dr, dc->security_store_filtered);
+		display_security_dialog (dc);
+	}
+	else
+		display_edit_friends_dialog (dc);
 	
 	return;
 }
