@@ -19,13 +19,15 @@
  * 		Todd Kulesza <todd@dropline.net>
  */
 
+#include <config.h>
+
+#include <stdarg.h> /* variable-length functions */
 #include <gnome.h>
 #include <gdk-pixbuf/gdk-pixdata.h>
 #include <glade/glade.h>
 #include <string.h>
 #include <curl/curl.h>
-
-#include <config.h>
+#include <gtksourceview/gtksourceview.h>
 
 #include "drivel_request.h"
 #include "blog_lj.h"
@@ -34,6 +36,16 @@
 #include "network.h"
 #include "dialogs.h"
 
+typedef struct _DrivelSpellLanguage DrivelSpellLanguage;
+
+/* realname, the name that aspell understands, i.e. en_US, en_GB etc.
+   label, the name displayed to the user, fallbacks to realname */
+struct _DrivelSpellLanguage
+{
+	gchar *realname;
+	gchar *label;
+};
+
 void
 history_list_edit_cb (GtkWidget *button, gpointer data);
 
@@ -74,7 +86,7 @@ dialog_grey_button_on_invalid (GtkWidget *button, GtkWidget *entry, ...)
 	GtkWidget *w;
 	DrivelButtonVAList *bval;
 	
-	args = g_new (va_list, 1);
+	args = g_new0 (va_list, 1);
 	
 	va_start (*args, entry);
 
@@ -120,7 +132,8 @@ mark_history_list (DrivelClient *dc)
 {
 	DrivelRequest *dr;
 	
-	dr = blog_lj_build_getdaycounts_request (dc->username, dc->active_journal);
+	dr = blog_lj_build_getdaycounts_request (dc->user->username, 
+			dc->user->server, dc->active_journal);
 	net_enqueue_request (dc, dr);
 	
 	return;
@@ -170,31 +183,11 @@ history_response_cb (GtkWidget *widget, gint arg, gpointer data)
 static void
 preferences_close_cb (GtkWidget *widget, gint arg, gpointer data)
 {
-	gchar *text;
-	gboolean close = TRUE;
 	DrivelClient *dc = (DrivelClient *) data;
 	
 	if (arg == GTK_RESPONSE_CLOSE || arg == GTK_RESPONSE_DELETE_EVENT)
 	{
-		if (GTK_WIDGET_IS_SENSITIVE (dc->pref_server))
-		{
-			text = gtk_editable_get_chars (GTK_EDITABLE (dc->pref_server),
-					0, -1);
-			if (!text || !strncmp (text, "", strlen (text)))
-			{
-				display_error_dialog (dc, _("Invalid journal server"),
-						_("Please enter a journal server address."));
-				close = FALSE;
-			}
-			
-			g_free (text);
-		}
-
-		if (close)
-		{
-			gtk_widget_destroy (widget);
-		}
-		
+		gtk_widget_destroy (widget);
 		drivel_pop_current_window (dc);
 	}
 	
@@ -264,7 +257,6 @@ update_friends_list (DrivelClient *dc)
 				-1);
 	}
 	
-	gtk_widget_set_sensitive (dc->friend_edit, FALSE);
 	gtk_widget_set_sensitive (dc->friend_remove, FALSE);
 	
 	return;
@@ -285,8 +277,9 @@ update_history_list_cb (GtkWidget *widget, gpointer data)
 			&year, &month, &day);
 	month++; /* the months are numbered starting with 0 */
 	
-	dr = blog_lj_build_getevents_request (dc->username, 50, TRUE, TRUE, "day",
-			NULL, year, month, day, 0, NULL, 0, dc->active_journal);
+	dr = blog_lj_build_getevents_request (dc->user->username, dc->user->server,
+			50, TRUE, TRUE, "day", NULL, year, month, day, 0, NULL, 0, 
+			dc->active_journal);
 	
 	net_enqueue_request (dc, dr);
 	
@@ -342,6 +335,66 @@ update_history_list (DrivelClient *dc, gchar **itemid, gchar **event, gchar **ev
 	return;
 }
 
+#ifdef HAVE_GTKSPELL
+
+static GSList *
+get_available_spell_languages (void)
+{
+	GSList *rv = NULL;
+	gchar *prgm, *cmd;
+	gchar *prgm_err, *prgm_out;
+	gint exit_status, i;
+	GError *err = NULL;
+	DrivelSpellLanguage *lang;
+	gchar **lang_arr;
+	
+	if ((prgm = g_find_program_in_path ("aspell")) == NULL)
+		return NULL;
+	cmd = g_strdup_printf ("%s dump dicts", prgm);
+	g_spawn_command_line_sync (cmd, &prgm_out, &prgm_err, &exit_status, &err);
+	g_free (cmd);
+	g_free (prgm);
+	if (err)
+	{
+		g_warning ("Failed to get language list: %s", err->message);
+		g_error_free (err);
+		return NULL;
+	}
+	if (exit_status != 0)
+	{
+		g_warning ("Failed to get language list, program output was: %s", prgm_err);
+		g_free (prgm_out);
+		g_free (prgm_err);
+		return NULL;
+	}
+	lang_arr = g_strsplit (prgm_out, "\n", -1);
+	
+	i = 0;
+	while (lang_arr[i])
+	{
+		g_strstrip (lang_arr[i]);
+		if (*(lang_arr[i]) != '\0')
+		{
+			lang = g_new0 (DrivelSpellLanguage, 1);
+			/* For now, set realname == label */
+			lang->realname = g_strdup (lang_arr[i]);
+			lang->label = g_strdup (lang_arr[i]);
+			rv = g_slist_insert_sorted (rv, lang, (GCompareFunc) g_strcasecmp);
+		}
+		i++;
+	}
+	g_strfreev (lang_arr);
+
+	lang = g_new0 (DrivelSpellLanguage, 1);
+	/* Context: Spell check dictionary */
+	lang->label = g_strdup (_("System default"));
+	lang->realname = g_strdup ("");
+	rv = g_slist_prepend (rv, lang);
+	
+	return rv;
+}
+#endif /* HAVE_GTKSPELL */
+
 static void
 add_friend_dialog_cb (GtkWidget *widget, gint arg, gpointer data)
 {
@@ -373,8 +426,8 @@ add_friend_dialog_cb (GtkWidget *widget, gint arg, gpointer data)
 			return;
 		}
 		
-		dr = blog_lj_build_editfriends_request (dc->username, friend, FALSE,
-				TRUE, fg, bg);
+		dr = blog_lj_build_editfriends_request (dc->user->username, 
+				dc->user->server, friend, FALSE, TRUE, fg, bg);
 		
 		net_enqueue_request (dc, dr);
 		
@@ -407,8 +460,8 @@ edit_friend_colour_cb (GtkWidget *widget, gpointer data)
 	bg = g_strdup_printf ("#%02X%02X%02X", 
 			color.red / 256, color.green / 256, color.blue / 256);
 	
-	dr = blog_lj_build_editfriends_request (dc->username, friend, FALSE,
-			FALSE, fg, bg);
+	dr = blog_lj_build_editfriends_request (dc->user->username, 
+			dc->user->server, friend, FALSE, FALSE, fg, bg);
 
 	net_enqueue_request (dc, dr);
 
@@ -437,100 +490,6 @@ sort_type_func (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer da
 	return retval;
 }
 
-static void
-entry_set_string (GConfClient *client, gchar *key, GtkWidget *entry)
-{
-	gchar *text;
-	text = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
-
-	gconf_client_set_string (client, key, text, NULL);
-	
-	g_free (text);
-
-	return;
-}
-
-static void
-server_list_cb (GtkWidget *menu, gpointer data)
-{
-	gint item;
-	gchar *server;
-	DrivelClient *dc = (DrivelClient *) data;
-	
-	item = gtk_combo_box_get_active (GTK_COMBO_BOX (menu));
-	
-	switch (item)
-	{
-		case 0:
-		{
-			server = g_strdup ("www.livejournal.com");
-			break;
-		}
-		case 1:
-		{
-			server = g_strdup ("www.deadjournal.com");
-			break;
-		}
-		case 2:
-		{
-			server = g_strdup ("www.ujournal.org");
-			break;
-		}
-		case 3:
-		{
-			server = g_strdup ("www.gotdiary.com");
-			break;
-		}
-		default:
-		{
-			gtk_widget_set_sensitive (dc->pref_server, TRUE);
-/*			gtk_widget_set_sensitive (dc->pref_server_label, TRUE); */
-			server = g_strdup ("");
-		}
-	}
-	if (item < 4)
-	{
-		gtk_widget_set_sensitive (dc->pref_server, FALSE);
-/*		gtk_widget_set_sensitive (dc->pref_server_label, FALSE); */
-		gconf_client_set_string (dc->client, dc->gconf->default_server, server, NULL);
-	}
-	
-	gtk_entry_set_text (GTK_ENTRY (dc->pref_server), server);
-	gconf_client_set_int (dc->client, dc->gconf->default_server_list, item, NULL);
-	
-	return;
-}
-
-static void
-server_cb (GtkWidget *entry, gpointer data)
-{
-	DrivelClient *dc = (DrivelClient *) data;
-
-	entry_set_string (dc->client, dc->gconf->default_server, entry);
-
-	return;
-}
-
-static gboolean
-server_focus_cb (GtkWidget *entry, GdkEventFocus *event, gpointer data)
-{
-	gchar *text;
-	DrivelClient *dc = (DrivelClient *) data;
-
-	text = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
-	if (!text || !strncmp (text, "", strlen (text)))
-	{
-/*		display_error_dialog (dc, _("Please enter the address of a journal server."));
-		gtk_widget_grab_focus (entry); */
-	}
-	else
-		entry_set_string (dc->client, dc->gconf->default_server, entry);
-
-	g_free (text);
-	
-	return FALSE;
-}
-
 static void
 tray_cb (GtkWidget *button, gpointer data)
 {
@@ -544,6 +503,19 @@ tray_cb (GtkWidget *button, gpointer data)
 	return;
 }
 
+static void
+technorati_cb (GtkWidget *button, gpointer data)
+{
+	gboolean state;
+	DrivelClient *dc = (DrivelClient *) data;
+	
+	state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
+	
+	gconf_client_set_bool (dc->client, dc->gconf->technorati, state, NULL);
+	
+	return;
+}
+
 static void
 min_start_cb (GtkWidget *button, gpointer data)
 {
@@ -596,49 +568,30 @@ spellcheck_cb (GtkWidget *button, gpointer user_data)
 
 	return;
 }
+
+static void
+spellcheck_language_cb (GtkComboBox *combobox, gpointer user_data)
+{
+	gchar *language;
+	DrivelClient *dc = (DrivelClient *) user_data;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	gint sel;
+
+	model = gtk_combo_box_get_model (combobox);
+	sel = gtk_combo_box_get_active (combobox);
+
+	if (gtk_tree_model_iter_nth_child (model, &iter, NULL, sel))
+	{
+		gtk_tree_model_get (model, &iter, 0, &language, -1);
+		gconf_client_set_string (dc->client, dc->gconf->spell_language, language, NULL);
+		g_free (language);
+	}
+
+	return;
+}
 #endif /* HAVE_GTKSPELL */
 
-static void
-mood_cb (GtkWidget *entry, gpointer data)
-{
-	DrivelClient *dc = (DrivelClient *) data;
-	
-	entry_set_string (dc->client, dc->gconf->default_mood, 
-			GTK_BIN (dc->preferences_mood)->child);
-	
-	return;
-}
-
-static gboolean
-mood_focus_cb (GtkWidget *entry, GdkEventFocus *event, gpointer data)
-{
-	DrivelClient *dc = (DrivelClient *) data;
-
-	entry_set_string (dc->client, dc->gconf->default_mood, entry);
-	
-	return FALSE;
-}
-
-static void
-music_cb (GtkWidget *entry, gpointer data)
-{
-	DrivelClient *dc = (DrivelClient *) data;
-	
-	entry_set_string (dc->client, dc->gconf->default_music, entry);
-	
-	return;
-}
-
-static gboolean
-music_focus_cb (GtkWidget *entry, GdkEventFocus *event, gpointer data)
-{
-	DrivelClient *dc = (DrivelClient *) data;
-	
-	entry_set_string (dc->client, dc->gconf->default_music, entry);
-	
-	return FALSE;
-}
-
 static void
 picture_cb (GtkWidget *menu, gpointer data)
 {
@@ -655,12 +608,36 @@ picture_cb (GtkWidget *menu, gpointer data)
 static void
 security_cb (GtkWidget *menu, gpointer data)
 {
-	gint number;
+	gint num;
+	gchar *name, *group;
+	GtkTreeIter iter;
 	DrivelClient *dc = (DrivelClient *) data;
 
-	number = gtk_combo_box_get_active (GTK_COMBO_BOX (menu));
+	gtk_combo_box_get_active_iter (GTK_COMBO_BOX (menu), &iter);
+	gtk_tree_model_get (gtk_combo_box_get_model (GTK_COMBO_BOX (menu)), &iter,
+			STORE_SECURITY_NAME, &name, STORE_SECURITY_NUM, &num, -1);
 
-	gconf_client_set_int (dc->client, dc->gconf->default_security, number, NULL);
+	if (name && !strcmp (name, _("Public")))
+	{
+		group = g_strdup ("public");
+		num = 0;
+	}
+	else if (name && !strcmp (name, _("Private")))
+	{
+		group = g_strdup ("private");
+		num = 0;
+	}
+	else
+	{
+		group = g_strdup ("usemask");
+		num = 1 << num;
+	}
+	
+	gconf_client_set_string (dc->client, dc->gconf->default_security, group, NULL);
+	gconf_client_set_int (dc->client, dc->gconf->default_security_mask, num, NULL);
+	
+	g_free (name);
+	g_free (group);
 	
 	return;
 }
@@ -726,14 +703,17 @@ history_list_edit_cb (GtkWidget *button, gpointer data)
 	
 	dc->edit_entry = TRUE;
 	
+	gtk_source_buffer_begin_not_undoable_action (GTK_SOURCE_BUFFER (dc->buffer));
+	
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (dc->history_list));
 	if (gtk_tree_selection_get_selected (selection, &model, &iter))
 	{
 		gtk_tree_model_get (model, &iter, HISTORY_ITEMID_COL, &itemid_string, -1);
 		itemid = g_strtod (itemid_string, NULL);
 
-		dr = blog_lj_build_getevents_request (dc->username, 0, FALSE, FALSE, "one",
-				NULL, 0, 0, 0, 0, NULL, itemid, dc->active_journal);
+		dr = blog_lj_build_getevents_request (dc->user->username, 
+				dc->user->server, 0, FALSE, FALSE, "one", NULL, 0, 0, 0, 0, 
+				NULL, itemid, dc->active_journal);
 	
 		net_enqueue_request (dc, dr);
 	}
@@ -970,8 +950,8 @@ friends_list_remove_cb (GtkWidget *widget, gpointer data)
 	gtk_tree_selection_get_selected (selection, &model, &iter);
 	gtk_tree_model_get (model, &iter, USERNAME_COL, &friend, -1);
 
-	dr = blog_lj_build_editfriends_request (dc->username, friend, TRUE, FALSE,
-			NULL, NULL);
+	dr = blog_lj_build_editfriends_request (dc->user->username, 
+			dc->user->server, friend, TRUE, FALSE, NULL, NULL);
 	
 	net_enqueue_request (dc, dr);
 	
@@ -1072,7 +1052,7 @@ image_response_cb (GtkWidget *widget, gint arg, gpointer data)
 	if (arg == GTK_RESPONSE_OK)
 	{
 		gchar *protocol, *prefix;
-
+		
 		url = gtk_editable_get_chars (GTK_EDITABLE (dc->image_url), 0, -1);
 		alt = gtk_editable_get_chars (GTK_EDITABLE (dc->image_alt), 0, -1);
 		
@@ -1081,25 +1061,27 @@ image_response_cb (GtkWidget *widget, gint arg, gpointer data)
 		{
 			prefix = g_strdup (url);
 			prefix[strlen (url) - strlen (protocol)] = '\0';
+			g_free (url);
+			url = g_strdup (protocol + 3);
 		}
 		else
 			prefix = g_strdup ("http");
-
+		
 		height = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (dc->image_height));
 		width = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (dc->image_width));
 		
 		if (height)
-			height_text = g_strdup_printf ("height=\"%d\"", height);
+			height_text = g_strdup_printf ("height=%d ", height);
 		else
 			height_text = g_strdup ("");
 		
 		if (width)
-			width_text = g_strdup_printf ("width=\"%d\"", width);
+			width_text = g_strdup_printf ("width=%d ", width);
 		else
 			width_text = g_strdup ("");
 		
 		image_text = g_strdup_printf (
-				"<img src=\"%s://%s\" %s %s alt=\"%s\">",
+				"<img src=\"%s://%s\" %s%salt=\"%s\" />",
 				prefix, url, height_text, width_text, alt);
 		
 		gtk_text_buffer_insert_at_cursor (dc->buffer, image_text, -1);
@@ -1110,7 +1092,7 @@ image_response_cb (GtkWidget *widget, gint arg, gpointer data)
 		g_free (height_text);
 		g_free (width_text);
 		g_free (prefix);
-		
+
 		gtk_widget_destroy (widget);
 	}
 	
@@ -1354,7 +1336,7 @@ display_edit_history_dialog (DrivelClient *dc)
 		return;
 	}
 
-	xml = load_glade_xml ();
+	xml = load_glade_xml ("history_dialog");
 	if (!xml)
 		return;
 	
@@ -1444,7 +1426,7 @@ display_edit_friends_dialog (DrivelClient *dc)
 		return;
 	}
 
-	xml = load_glade_xml ();
+	xml = load_glade_xml ("friends_dialog");
 	if (!xml)
 		return;
 	
@@ -1568,7 +1550,7 @@ display_insert_link_dialog (DrivelClient *dc)
 		return;
 	}
 
-	xml = load_glade_xml ();
+	xml = load_glade_xml ("insert_link_dialog");
 	if (!xml)
 		return;
 
@@ -1656,7 +1638,7 @@ display_insert_image_dialog (DrivelClient *dc)
 		return;
 	}
 	
-	xml = load_glade_xml ();
+	xml = load_glade_xml ("insert_image_dialog");
 	if (!xml)
 		return;
 
@@ -1700,25 +1682,106 @@ display_insert_image_dialog (DrivelClient *dc)
 	return;
 }
 
+static void
+security_group_selection_changed_cb (GtkTreeSelection *selection, gpointer data)
+{
+	
+	return;
+}
+
+void
+display_security_dialog (DrivelClient *dc)
+{
+	GtkCellRenderer *renderer;
+	GtkTreeViewColumn *column;
+	GtkTreeSelection *selection;
+	GtkWidget *widget;
+	GladeXML *xml;
+	static GtkWidget *dialog = NULL;
+	
+	if (dialog)
+	{
+		gtk_window_present (GTK_WINDOW (dialog));
+		
+		return;
+	}
+	
+	xml = load_glade_xml ("security_dialog");
+	if (!xml)
+		return;
+	
+	dialog = glade_xml_get_widget (xml, "security_dialog");
+	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (dc->current_window));
+	
+	/* Friends Groups list*/
+	widget = glade_xml_get_widget (xml, "security_list");
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
+	g_signal_connect (G_OBJECT (selection), "changed",
+			G_CALLBACK (security_group_selection_changed_cb), dc);
+	gtk_tree_view_set_model (GTK_TREE_VIEW (widget), 
+			GTK_TREE_MODEL (dc->security_store_filtered));
+	renderer = gtk_cell_renderer_text_new ();
+	column = gtk_tree_view_column_new_with_attributes (_("Available"),
+			renderer, "text", STORE_SECURITY_NAME, NULL);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (widget), column);
+	
+	/* Available users list */
+	widget = glade_xml_get_widget (xml, "security_details_friends_available");
+	renderer = gtk_cell_renderer_text_new ();
+	column = gtk_tree_view_column_new_with_attributes (_("Available"),
+			renderer, "text", STORE_SECURITY_NAME, NULL);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (widget), column);
+	
+	/* In-Group users list */
+	widget = glade_xml_get_widget (xml, "security_details_friends_in_group");
+	renderer = gtk_cell_renderer_text_new ();
+	column = gtk_tree_view_column_new_with_attributes (_("In Group"),
+			renderer, "text", STORE_SECURITY_NAME, NULL);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (widget), column);
+	
+	g_signal_connect (G_OBJECT (dialog), "destroy",
+			G_CALLBACK (gtk_widget_destroyed), &dialog);
+	g_signal_connect (G_OBJECT (dialog), "response",
+			G_CALLBACK (gtk_widget_destroy), dialog);
+	
+	gtk_widget_show_all (dialog);
+	
+	return;
+}
+
+static void
+free_store (GtkWidget *widget, GObject *store)
+{
+	g_object_unref (store);
+	
+	return;
+}
+
+static void
+nullify_pointer (GtkWidget *widget, gpointer *user_data)
+{
+	*user_data = NULL;
+
+	return;
+}
+
 void
 display_edit_preferences_dialog (DrivelClient *dc)
 {
 	GladeXML *xml;
 	static GtkWidget *dialog = NULL;
-	GtkWidget *parent;
-	GtkWidget *min_post, *min_start, *tray, *highlight_syntax, *spellcheck;
-	GtkWidget *server_list, *server, *mood, *music, *security, *picture;
-	GtkWidget *comments, *autoformat;
-	GtkWidget *tempwidget;
-	GtkListStore *security_store, *picture_store, *server_store, *mood_store;
+	GtkWidget *widget, *spellcheck;
+	GtkListStore *picture_store;
 	GtkTreeIter iter;
 	GtkCellRenderer *renderer;
 	gboolean state;
-	gint i;
 	gchar *string;
+	gint num;
+#ifdef HAVE_GTKSPELL
+	gint sel = -1;
+	GtkListStore *language_store;
+#endif /* HAVE_GTKSPELL */
 	GSList *list;
-	GdkPixbuf *pixbuf;
-	GtkEntryCompletion *completion;
 	GConfValue *value;
 
 	if (dialog)
@@ -1727,226 +1790,226 @@ display_edit_preferences_dialog (DrivelClient *dc)
 		return;
 	}
 
-	xml = load_glade_xml ();
+	xml = load_glade_xml ("preferences_dialog");
 	if (!xml)
 		return;
-	
+
 	/* dialog box */
 	dialog = glade_xml_get_widget (xml, "preferences_dialog");
-	parent = drivel_get_current_window (dc->window_list);
-	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (parent));
-	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (dc->current_window));
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
 	drivel_push_current_window (dc, dialog);
 	
 	/* tray icon */
-	tray = glade_xml_get_widget (xml, "pref_general_notification");
-	state = gconf_client_get_bool (dc->client, dc->gconf->tray, NULL);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tray), state);
-	g_signal_connect (G_OBJECT (tray), "toggled",
+	widget = glade_xml_get_widget (xml, "pref_general_notification");
+	value = gconf_client_get (dc->client, dc->gconf->tray, NULL);
+	if (value)
+		state = gconf_value_get_bool (value);
+	else
+		state = TRUE;
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), state);
+	g_signal_connect (G_OBJECT (widget), "toggled",
 			G_CALLBACK (tray_cb), dc);
 
+	/* technorati */
+	widget = glade_xml_get_widget (xml, "pref_general_technorati");
+	value = gconf_client_get (dc->client, dc->gconf->technorati, NULL);
+	if (value)
+		state = gconf_value_get_bool (value);
+	else
+		state = FALSE;
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), state);
+	g_signal_connect (G_OBJECT (widget), "toggled",
+			G_CALLBACK (technorati_cb), dc);
+	
 	/* minimise after logging in */
-	min_start = glade_xml_get_widget (xml, "pref_general_entry_minimizelogin");
+	widget = glade_xml_get_widget (xml, "pref_general_entry_minimizelogin");
 	state = gconf_client_get_bool (dc->client, dc->gconf->min_start, NULL);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (min_start), state);
-	g_signal_connect (G_OBJECT (min_start), "toggled",
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), state);
+	g_signal_connect (G_OBJECT (widget), "toggled",
 			G_CALLBACK (min_start_cb), dc);
 
 	/* minimise after posting */
-	min_post = glade_xml_get_widget (xml, "pref_general_entry_minimizepost");
+	widget = glade_xml_get_widget (xml, "pref_general_entry_minimizepost");
 	state = gconf_client_get_bool (dc->client, dc->gconf->min_post, NULL);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (min_post), state);
-	g_signal_connect (G_OBJECT (min_post), "toggled",
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), state);
+	g_signal_connect (G_OBJECT (widget), "toggled",
 			G_CALLBACK (min_post_cb), dc);
 
 	/* syntax highlighting */
-	highlight_syntax = glade_xml_get_widget (xml, "pref_general_entry_highlightsyntax");
+	widget = glade_xml_get_widget (xml, "pref_general_entry_highlightsyntax");
 	value = gconf_client_get (dc->client, dc->gconf->highlight_syntax, NULL);
 	if (value)
 		state = gconf_value_get_bool (value);
 	else
 		state = TRUE;
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (highlight_syntax), state);
-	g_signal_connect (G_OBJECT (highlight_syntax), "toggled",
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), state);
+	g_signal_connect (G_OBJECT (widget), "toggled",
 			G_CALLBACK (highlight_syntax_cb), dc);
 
-	/* spellcheck */
-	spellcheck = glade_xml_get_widget (xml, "pref_general_entry_spellcheck");
+	/* generic spellcheck widgets */
+	
+	widget = glade_xml_get_widget (xml, "pref_general_entry_spellcheck");
+	spellcheck = widget;
+	widget = glade_xml_get_widget (xml, "prefs_general_entry_dictionary_box");
+	dc->pref_dictionary_box = widget;
+	widget = glade_xml_get_widget (xml, "prefs_general_entry_dictionary");
+	dc->pref_dictionary = widget;
+
 #ifdef HAVE_GTKSPELL
+	
+	/* spellcheck */
+	
 	value = gconf_client_get (dc->client, dc->gconf->spellcheck, NULL);
 	if (value)
 		state = gconf_value_get_bool (value);
 	else
 		state = FALSE;
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (spellcheck), state);
+	gtk_widget_set_sensitive (dc->pref_dictionary_box, state);
 	g_signal_connect (G_OBJECT (spellcheck), "toggled",
 			G_CALLBACK (spellcheck_cb), dc);
-#endif /* HAVE_GTKSPELL */
+
+	/* spellcheck dictionaries */
+
+	string = get_default_text (dc->client, dc->gconf->spell_language, "");
 	
-	/* server list */
-	server_store = gtk_list_store_new (1, G_TYPE_STRING);
-	gtk_list_store_append (server_store, &iter);
-	gtk_list_store_set (server_store, &iter, 0, ("LiveJournal"), -1);
-	gtk_list_store_append (server_store, &iter);
-	gtk_list_store_set (server_store, &iter, 0, ("DeadJournal"), -1);
-	gtk_list_store_append (server_store, &iter);
-	gtk_list_store_set (server_store, &iter, 0, ("uJournal"), -1);
-	gtk_list_store_append (server_store, &iter);
-	gtk_list_store_set (server_store, &iter, 0, ("GotDiary?"), -1);
-	gtk_list_store_append (server_store, &iter);
-	gtk_list_store_set (server_store, &iter, 0, _("Custom..."), -1);
-	server_list = glade_xml_get_widget (xml, "pref_network_server_combo");
-	gtk_combo_box_set_model (GTK_COMBO_BOX (server_list), GTK_TREE_MODEL (server_store));
-	gtk_cell_layout_clear (GTK_CELL_LAYOUT (server_list));
-	renderer = gtk_cell_renderer_text_new ();
-	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (server_list), renderer, TRUE);
-	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (server_list), renderer,
-			"text", 0,
-			NULL);
-	i = gconf_client_get_int (dc->client, dc->gconf->default_server_list, NULL);
-	gtk_combo_box_set_active (GTK_COMBO_BOX (server_list), i);
-	g_signal_connect (G_OBJECT (server_list), "changed",
-			G_CALLBACK (server_list_cb), dc);
-	server = glade_xml_get_widget (xml, "pref_network_server_address");
-	string = get_default_text (dc->client, dc->gconf->default_server, "www.livejournal.com");
-	gtk_entry_set_text (GTK_ENTRY (server), string);
-	g_free (string);
-	g_signal_connect (G_OBJECT (server), "focus_out_event",
-			G_CALLBACK (server_focus_cb), dc);
-	g_signal_connect (G_OBJECT (server), "activate",
-			G_CALLBACK (server_cb), dc);
-	if (i < 3)
-		gtk_widget_set_sensitive (server, FALSE);
-	dc->pref_server = server;
-/*	dc->pref_server_label = glade_xml_get_widget (xml, "pref_network_server_address_label"); */
-
-	/* network options */
-	/* FIXME: this isn't implemented yet */
-	tempwidget = glade_xml_get_widget (xml, "pref_network_options_2");
-	gtk_widget_set_sensitive (tempwidget, FALSE);
-
-	/* mood */
-	mood_store = gtk_list_store_new (1, G_TYPE_STRING);
-	if (dc->mood_list)
+	language_store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
+	list = get_available_spell_languages ();
+	if (list)
 	{
-		dc->mood_list = g_slist_sort (dc->mood_list, string_compare);
-		for (list = dc->mood_list; list != NULL; list = list->next)
+		DrivelSpellLanguage *lang_struct;
+		gint i;
+		GSList *head = list;
+		
+		for (i = 0, list = head; list != NULL; list = g_slist_next (list), i++)
 		{
-			gtk_list_store_append (mood_store, &iter);
-			gtk_list_store_set (mood_store, &iter, 0, list->data, -1);
+			lang_struct = list->data;
+			gtk_list_store_append (language_store, &iter);
+			gtk_list_store_set (language_store, &iter,
+					    0, lang_struct->realname,
+					    1, lang_struct->label,
+					    -1);
+			if (g_str_equal (lang_struct->realname, string))
+				sel = i;
+			g_free (lang_struct->realname);
+			g_free (lang_struct->label);
 		}
+		
+		g_slist_free (head);
 	}
-	mood = glade_xml_get_widget (xml, "pref_defaults_mood");
-	gtk_combo_box_set_model (GTK_COMBO_BOX (mood), GTK_TREE_MODEL (mood_store));
-	gtk_cell_layout_clear (GTK_CELL_LAYOUT (mood));
+	gtk_combo_box_set_model (GTK_COMBO_BOX (widget), GTK_TREE_MODEL (language_store));
+ 	gtk_cell_layout_clear (GTK_CELL_LAYOUT (widget));
 	renderer = gtk_cell_renderer_text_new ();
-	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (mood), renderer, TRUE);
-	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (mood), renderer,
-			"text", 0,
-			NULL);
-	string = get_default_text (dc->client, dc->gconf->default_mood, "");
-	gtk_entry_set_text (GTK_ENTRY (GTK_BIN (mood)->child), string);
-	g_free (string);
-	g_signal_connect (G_OBJECT (GTK_BIN (mood)->child), "focus_out_event",
-			G_CALLBACK (mood_focus_cb), dc);
-	g_signal_connect (G_OBJECT (GTK_COMBO_BOX (mood)), "changed",
-			G_CALLBACK (mood_cb), dc);
-	completion = gtk_entry_completion_new ();
-	gtk_entry_set_completion (GTK_ENTRY (GTK_BIN (mood)->child), completion);
-	gtk_entry_completion_set_model (completion, GTK_TREE_MODEL (mood_store));
-	gtk_entry_completion_set_text_column (completion, 0);
-	dc->preferences_mood = mood;
-
-	/* music */
-	music = glade_xml_get_widget (xml, "pref_defaults_music");
-	string = get_default_text (dc->client, dc->gconf->default_music, "");
-	gtk_entry_set_text (GTK_ENTRY (music), string);
-	g_free (string);
-	g_signal_connect (G_OBJECT (music), "focus_out_event",
-			G_CALLBACK (music_focus_cb), dc);
-	g_signal_connect (G_OBJECT (music), "activate",
-			G_CALLBACK (music_cb), dc);
-
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (widget), renderer, TRUE);
+	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (widget), renderer,
+					"text", 1,
+					NULL);
+	if (sel < 0)
+	{
+		if (strcmp (string, ""))
+			g_warning ("Language \"%s\" from GConf isn't in the list of available languages\n", string);
+		sel = 0;
+	}
+	
+	gtk_combo_box_set_active (GTK_COMBO_BOX (widget), sel);
+	g_signal_connect (G_OBJECT (widget), "changed",
+			  G_CALLBACK (spellcheck_language_cb), dc);
+	g_signal_connect (G_OBJECT (dialog), "destroy",
+			  G_CALLBACK (free_store), G_OBJECT (language_store));
+#endif /* HAVE_GTKSPELL */
+	g_signal_connect (G_OBJECT (dialog), "destroy",
+			  G_CALLBACK (nullify_pointer), &(dc->pref_dictionary));
+	
 	/* picture */
 	picture_store = gtk_list_store_new (2, GDK_TYPE_PIXBUF, G_TYPE_STRING);
 	fill_picture_menu (dc, picture_store);
-	picture = glade_xml_get_widget (xml, "pref_defaults_picture");
-	gtk_combo_box_set_model (GTK_COMBO_BOX (picture), GTK_TREE_MODEL (picture_store));
-	gtk_cell_layout_clear (GTK_CELL_LAYOUT (picture));
+	widget = glade_xml_get_widget (xml, "pref_defaults_picture");
+	gtk_combo_box_set_model (GTK_COMBO_BOX (widget), GTK_TREE_MODEL (picture_store));
+	gtk_cell_layout_clear (GTK_CELL_LAYOUT (widget));
 	if (!dc->picture_keywords)
-		gtk_widget_set_sensitive (picture, FALSE);
-	gtk_combo_box_set_active (GTK_COMBO_BOX (picture),
+		gtk_widget_set_sensitive (widget, FALSE);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (widget),
 			gconf_client_get_int (dc->client, dc->gconf->default_picture, NULL));
 	renderer = gtk_cell_renderer_pixbuf_new ();
-	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (picture), renderer, FALSE);
-	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (picture), renderer,
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (widget), renderer, FALSE);
+	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (widget), renderer,
 			"pixbuf", 0,
 			NULL);
 	renderer = gtk_cell_renderer_text_new ();
-	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (picture), renderer, TRUE);
-	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (picture), renderer,
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (widget), renderer, TRUE);
+	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (widget), renderer,
 			"text", 1,
 			NULL);
-	g_signal_connect (G_OBJECT (picture), "changed",
+	g_signal_connect (G_OBJECT (widget), "changed",
 			G_CALLBACK (picture_cb), dc);
-
+	g_signal_connect (G_OBJECT (dialog), "destroy",
+			G_CALLBACK (free_store), G_OBJECT (picture_store));
+	
 	/* security */
-	security_store = gtk_list_store_new (2, GDK_TYPE_PIXBUF, G_TYPE_STRING);
-	gtk_list_store_append (security_store, &iter);
-	pixbuf = gdk_pixbuf_new_from_file (DATADIR G_DIR_SEPARATOR_S "pixmaps"
-			G_DIR_SEPARATOR_S "drivel" G_DIR_SEPARATOR_S "public.png", NULL);
-	gtk_list_store_set (security_store, &iter, 0, pixbuf, 1, _("Public"), -1);
-	gtk_list_store_append (security_store, &iter);
-	pixbuf = gdk_pixbuf_new_from_file (DATADIR G_DIR_SEPARATOR_S "pixmaps"
-			G_DIR_SEPARATOR_S "drivel" G_DIR_SEPARATOR_S "protected.png", NULL);
-	gtk_list_store_set (security_store, &iter, 0, pixbuf, 1, _("Friends-Only"), -1);
-	gtk_list_store_append (security_store, &iter);
-	pixbuf = gdk_pixbuf_new_from_file (DATADIR G_DIR_SEPARATOR_S "pixmaps"
-			G_DIR_SEPARATOR_S "drivel" G_DIR_SEPARATOR_S "private.png", NULL);
-	gtk_list_store_set (security_store, &iter, 0, pixbuf, 1, _("Private"), -1);
-	security = glade_xml_get_widget (xml, "pref_defaults_security");
-	gtk_combo_box_set_model (GTK_COMBO_BOX (security), GTK_TREE_MODEL (security_store));
-	gtk_cell_layout_clear (GTK_CELL_LAYOUT (security));
+	widget = glade_xml_get_widget (xml, "pref_defaults_security");
+	gtk_combo_box_set_model (GTK_COMBO_BOX (widget), 
+			GTK_TREE_MODEL (dc->security_store));
+	gtk_cell_layout_clear (GTK_CELL_LAYOUT (widget));
 	renderer = gtk_cell_renderer_pixbuf_new ();
-	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (security), renderer, FALSE);
-	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (security), renderer,
-			"pixbuf", 0,
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (widget), renderer, FALSE);
+	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (widget), renderer,
+			"pixbuf", STORE_SECURITY_ICON,
 			NULL);
 	renderer = gtk_cell_renderer_text_new ();
-	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (security), renderer, TRUE);
-	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (security), renderer,
-			"text", 1,
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (widget), renderer, TRUE);
+	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (widget), renderer,
+			"text", STORE_SECURITY_NAME,
 			NULL);
-	gtk_combo_box_set_active (GTK_COMBO_BOX (security),
-			gconf_client_get_int (dc->client, dc->gconf->default_security, NULL));
-	g_signal_connect (G_OBJECT (security), "changed",
-			G_CALLBACK (security_cb), dc);
+	if (dc->journal_security)
+	{
+		string = gconf_client_get_string (dc->client, dc->gconf->default_security, NULL);
+		num = gconf_client_get_int (dc->client, dc->gconf->default_security_mask, NULL);
+		select_security_group (GTK_TREE_MODEL (dc->security_store), 
+				GTK_COMBO_BOX (widget), string, num);
+		g_signal_connect (G_OBJECT (widget), "changed",
+				G_CALLBACK (security_cb), dc);
+		g_free (string);
+	}
+	else
+		gtk_widget_set_sensitive (widget, FALSE);
 
 	/* comments */
-	comments = glade_xml_get_widget (xml, "pref_defaults_nocomments");
+	widget = glade_xml_get_widget (xml, "pref_defaults_nocomments");
 	state = gconf_client_get_bool (dc->client, dc->gconf->default_comment, NULL);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (comments), state);
-	g_signal_connect (G_OBJECT (comments), "toggled",
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), state);
+	g_signal_connect (G_OBJECT (widget), "toggled",
 			G_CALLBACK (comments_cb), dc);
 
 	/* autoformat */
-	autoformat = glade_xml_get_widget (xml, "pref_defaults_noautoformat");
+	widget = glade_xml_get_widget (xml, "pref_defaults_noautoformat");
 	state = gconf_client_get_bool (dc->client, dc->gconf->default_autoformat, NULL);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (autoformat), state);
-	g_signal_connect (G_OBJECT (autoformat), "toggled",
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), state);
+	g_signal_connect (G_OBJECT (widget), "toggled",
 			G_CALLBACK (autoformat_cb), dc);
 
+	g_signal_connect_swapped (G_OBJECT (dialog), "destroy",
+			G_CALLBACK (g_object_unref), G_OBJECT (xml));
 	g_signal_connect (G_OBJECT (dialog), "destroy",
 			G_CALLBACK (gtk_widget_destroyed), &dialog);
 	g_signal_connect (G_OBJECT (dialog), "response",
 			G_CALLBACK (preferences_close_cb), dc);
 
+	/* The Entry Defaults tab (index of 1) only applies to LiveJournal */
+	if (dc->user->api != BLOG_API_LJ)
+	{
+		widget = glade_xml_get_widget (xml, "pref_notebook");
+		gtk_notebook_remove_page (GTK_NOTEBOOK (widget), 1);
+	}
+	
 	gtk_widget_show_all (dialog);
 	
+	
+	
 #ifndef HAVE_GTKSPELL
 	gtk_widget_hide (spellcheck);
+	gtk_widget_hide (dc->pref_dictionary_box);
 #endif
-
+	
 	return;
 }
