@@ -13,7 +13,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * Copyright 2002-2004 Todd Kulesza
+ * Copyright 2002-2005 Todd Kulesza
  *
  * Authors:
  * 		Todd Kulesza <todd@dropline.net>
@@ -22,19 +22,19 @@
 #ifndef _DRIVEL_H_
 #define _DRIVEL_H_
 
-#include "config.h"
 #include "drivel_request.h"
 #include "egg-recent.h"
 #include "utils.h"
 
+#define _XOPEN_SOURCE /* glibc2 needs this */
+#include <time.h>
 #include <gtk/gtk.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <gconf/gconf-client.h>
 #include <curl/curl.h>
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <netdb.h>
-#include <arpa/inet.h>
+
+#define DRIVEL_APPKEY "EB85AE01035D2303C7D70415B673CE9937BE4056DE691BF2"
+#define DRIVEL_N_RECENT_POSTS 15
 
 typedef struct _DrivelClient DrivelClient;
 typedef struct _DrivelNet DrivelNet;
@@ -44,6 +44,8 @@ typedef struct _DrivelTime DrivelTime;
 typedef struct _LJMenuItem LJMenuItem;
 typedef struct _LJFriend LJFriend;
 typedef struct _DrivelButtonVAList DrivelButtonVAList;
+typedef struct _DrivelJournalEntry DrivelJournalEntry;
+typedef struct _DrivelJournalProp DrivelJournalProp;
 
 struct _LJFriend
 {
@@ -92,12 +94,11 @@ struct _DrivelButtonVAList
 struct _DrivelIDs
 {
 	/* gconf ids */
-	guint server_list_id;
-	guint server_id;
 	guint mood_id;
 	guint music_id;
 	guint picture_id;
 	guint security_id;
+	guint security_mask_id;
 	guint comment_id;
 	guint autoformat_id;
 	guint proxy_id;
@@ -106,8 +107,12 @@ struct _DrivelIDs
 	guint proxy_user_id;
 	guint proxy_pass_id;
 	guint tray_id;
+	guint expander_open_id;
 	guint highlight_syntax_id;
 	guint spellcheck_id;
+	guint spell_language_id;
+	guint last_journal_id;
+	guint technorati_id;
 	
 	/* loop ids */
 	guint loop_checkfriends_id;
@@ -117,32 +122,26 @@ struct _DrivelGConfData
 {
 	/* login window preferences */
 	gchar *user;
-	gchar *password;
-	gchar *save_password;
-	gchar *autologin;
 	
-	/* journal window default preferences */
-	gchar *default_server_list;
-	gchar *default_server;
+	/* journal entry preferences */
 	gchar *default_mood;
 	gchar *default_music;
 	gchar *default_picture;
 	gchar *default_security;
+	gchar *default_security_mask;
 	gchar *default_comment;
 	gchar *default_autoformat;
 	gchar *expander_open;
-	
-	/* proxy preferencess */
-	gchar *proxy;
-	gchar *proxy_url;
-	gchar *proxy_port;
-	gchar *proxy_user;
-	gchar *proxy_pass;
+	gchar *highlight_syntax;
+	gchar *spellcheck;
+	gchar *spell_language;
+	gchar *last_journal;
+	gchar *technorati;
 	
 	/* notification area preferences */
 	gchar *tray;
 	
-	/* entry position and size */
+	/* entry window position and size */
 	gchar *entry_x;
 	gchar *entry_y;
 	gchar *entry_height;
@@ -150,14 +149,32 @@ struct _DrivelGConfData
 	gchar *entry_max;
 	gchar *min_start;
 	gchar *min_post;
-	gchar *highlight_syntax;
-	gchar *spellcheck;
+};
+
+struct _DrivelJournalEntry
+{
+	gchar *subject;
+	gchar *content;
+	struct tm date_posted;
+	gchar *security; /* lj security type */
+	gchar *security_mask; /* lj security mask */
+	gchar *postid; /* post_uri for atom, postid for others */
+	gchar *userid; /* feed_uri for atom, userid for others */
+	gchar *issued;
+	gchar *link;
+	GArray *properties; /* lj meta data */
+};
+
+struct _DrivelJournalProp
+{
+	gchar *id;
+	gchar *name;
+	gchar *value;
 };
 
 struct _DrivelNet
 {
 	DrivelBlogAPI api;
-	gchar *server;
 	CURLSH *curl_share;
 	gint fast_servers;
 	GAsyncQueue *msg_queue;
@@ -166,6 +183,7 @@ struct _DrivelNet
 
 struct _DrivelClient
 {
+	/* windows */
 	GtkWidget *login_window;
 	GtkWidget *journal_window;
 	GtkWidget *current_window;
@@ -182,8 +200,10 @@ struct _DrivelClient
 	GtkWidget *autologin;
 	GtkWidget *login_button;
 	GtkWidget *login_menu;
-	GtkWidget *sign_up_button;
-	GtkWidget *sign_up_menu;
+	GtkWidget *login_type;
+	GtkWidget *login_server;
+	GtkWidget *login_remove;
+	GtkListStore *login_type_store;
 	
 	/* journal window widgets */
 	GtkWidget *journal_subject;
@@ -191,19 +211,19 @@ struct _DrivelClient
 	GtkWidget *journal_music;
 	GtkWidget *journal_text;
 	GtkWidget *journal_security;
+	GtkWidget *journal_expander;
 	GtkWidget *journal_picture;
 	GtkWidget *journal_comment;
 	GtkWidget *journal_autoformat;
 	GtkWidget *journal_backdate;
 	GtkWidget *journal_date;
-	GtkWidget *options_frame;
+	GtkWidget *journal_category;
 	GtkTextBuffer *buffer;
-	GtkWidget *preferences_mood;
 	GtkWidget *journal_post;
-	GtkWidget *journal_options;
 	GtkWidget *edit_delete;
 	GtkWidget *edit_save;
 	GtkWidget *edit_cancel;
+	GtkUIManager *menus;
 	GtkWidget *menu_post;
 	GtkWidget *menu_update;
 	GtkWidget *menu_delete;
@@ -212,18 +232,21 @@ struct _DrivelClient
 	GtkWidget *menu_cut;
 	GtkWidget *menu_copy;
 	GtkWidget *menu_paste;
+	GtkWidget *menu_last;
+	GtkWidget *menu_friends;
+	GtkWidget *menu_history;
+	GtkWidget *menu_security;
+	GtkWidget *menu_active;
+	GtkAction *menu_view_options;
+	GtkAction *menu_view_misspelled_words;
+	GtkAction *menu_view_html_keywords;
 	EggRecentModel *recent_model;
-	EggRecentViewGtk *recent_view;
-	
-	/* preferences widgets */
-	GtkWidget *pref_server_list;
-	GtkWidget *pref_server;
-	GtkWidget *pref_server_label;
+	EggRecentViewUIManager *recent_view;
+	DrivelJournalEntry *journal_entry;
 	
 	/* friends dialog */
 	GtkListStore *list_store;
 	GtkWidget *friend_list;
-	GtkWidget *friend_edit;
 	GtkWidget *friend_remove;
 	GtkWidget *friends_username;
 	GtkWidget *friends_fg_colour;
@@ -260,27 +283,24 @@ struct _DrivelClient
 	GtkWidget *image_ok;
 	DrivelButtonVAList *image_bval;
 	
-	GtkWidget *message_dialog;
-	GtkWidget *open_dialog;
-	GtkWidget *save_dialog;
-	GtkWidget *network_progress;
-	
 	/* network data */
 	DrivelNet *net;
-	gchar *proxy_proper_format;
 	guint tag_autosave;
 	gint time_since_checkfriends;
 	gchar *lastupdate;
 	gboolean checking_friends;
 	gboolean friends_update;
+
+	/* preferences widgets */
+	GtkWidget *pref_dictionary;
+	GtkWidget *pref_dictionary_box;
 	
 	GConfClient *client;
 	DrivelGConfData *gconf;
 	
 	/* user-specific settings */
-	gchar *name;
-	gchar *username;
-	gchar *password;
+	DrivelUser *user;
+	GSList *user_list;
 	gboolean use_fast_servers;
 	GHashTable *picture_keywords;
 	GHashTable *picture_filenames;
@@ -291,33 +311,36 @@ struct _DrivelClient
 	gint moods;
 	gint journals;
 	GSList *journal_list;
-	gchar *active_journal;
-	gchar *itemid;
+	DrivelJournal *active_journal;
 	gboolean modified;
 	gboolean modified_autosave;
-	gint response;
 	gboolean proxy;
+	gboolean proxy_auth;
 	gchar *proxy_user;
 	gchar *proxy_pass;
 	gchar *proxy_url;
 	gint proxy_port;
 	GSList *menu_list;
 	GSList *friends_list;
-	gchar *login_send_data;
-	gboolean invalid_input;
 	gboolean edit_entry;
 	GSList *window_list;
-	gboolean tray;
 	GtkListStore *picture_store;
+	GtkListStore *category_store;
+	GtkTreeModel *category_store_filtered;
+	GtkListStore *security_store;
+	GtkListStore *security_store_filtered;
 	gchar *draft_filename;
-	
 	DrivelTime time;
 	DrivelIDs id;
+	GArray *recent_entries;
 };
 
 void
 display_error_dialog (DrivelClient *dc, const gchar *header, const gchar *mesg);
 
+void
+debug (const gchar *msg);
+
 void
 fill_time (DrivelClient *dc);
 
@@ -334,7 +357,8 @@ GtkWidget *
 drivel_get_current_window (GSList *list);
 
 void
-drivel_gconf_data_fill (DrivelGConfData *data, const gchar *username, GConfClient *client, DrivelIDs *id, DrivelClient *dc);
+drivel_gconf_data_fill (DrivelGConfData *data, const gchar *username, 
+		DrivelBlogAPI api, GConfClient *client, DrivelIDs *id, DrivelClient *dc);
 
 void
 drivel_fill_journal_null (DrivelClient *dc);
@@ -357,7 +381,19 @@ friends_list_free_item (gpointer data, gpointer user_data);
 gint
 compare_usernames (gconstpointer a, gconstpointer b);
 
+void
+fill_security_menu (DrivelClient *dc, GtkListStore *security_store);
+
 void
 fill_picture_menu (DrivelClient *dc, GtkListStore *picture_store);
 
+void
+add_account_to_list (DrivelClient *dc);
+
+gint
+sort_journals (DrivelJournal *a, DrivelJournal *b);
+
+void
+clear_recent_entries (GArray *recent);
+
 #endif
