@@ -13,7 +13,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * Copyright 2002-2004 Todd Kulesza
+ * Copyright 2002-2005 Todd Kulesza
  *
  * Authors:
  * 		Todd Kulesza <todd@dropline.net>
@@ -21,6 +21,8 @@
 
 #include <config.h>
 
+#include <math.h>
+#include <popt.h>
 #include <gnome.h>
 #include <gdk-pixbuf/gdk-pixdata.h>
 #include <libgnomevfs/gnome-vfs.h>
@@ -28,6 +30,7 @@
 #include <time.h>
 #include <string.h>
 #include <curl/curl.h>
+#include <libxml/parser.h>
 
 #include "drivel.h"
 #include "msg_queue.h"
@@ -35,7 +38,11 @@
 #include "network.h"
 #include "journal.h"
 
+/* size for picture icons */
+#define ICON_SIZE 20
+
 GMutex *net_mutex;
+gboolean verbose;
 
 void
 display_error_dialog (DrivelClient *dc, const gchar *header, const gchar *mesg)
@@ -57,16 +64,22 @@ display_error_dialog (DrivelClient *dc, const gchar *header, const gchar *mesg)
 	error_string = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">"
 			"%s</span>\n\n%s", error_header, error_mesg);
 	
-	dialog = gtk_message_dialog_new_with_markup (
-			GTK_WINDOW (dc->current_window),
-			GTK_DIALOG_DESTROY_WITH_PARENT,
-			GTK_MESSAGE_ERROR,
-			GTK_BUTTONS_CLOSE,
-			error_string);
+	if (dc->current_window)
+	{
+		dialog = gtk_message_dialog_new_with_markup (
+				GTK_WINDOW (dc->current_window),
+				GTK_DIALOG_DESTROY_WITH_PARENT,
+				GTK_MESSAGE_ERROR,
+				GTK_BUTTONS_CLOSE,
+				error_string);
 		
-	gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_dialog_run (GTK_DIALOG (dialog));
 		
-	gtk_widget_destroy (dialog);
+		gtk_widget_destroy (dialog);
+	}
+	else
+		g_warning ("%s:\n%s", error_header, error_mesg);
+	
 	g_free (error_header);
 	g_free (error_mesg);
 	g_free (error_string);
@@ -74,14 +87,106 @@ display_error_dialog (DrivelClient *dc, const gchar *header, const gchar *mesg)
 	return;
 }
 
-void fill_time(DrivelClient *dc)
+/* If the user request verbose output, display a debugging message */
+
+void
+debug (const gchar *msg)
+{
+	if (verbose)
+		g_print ("debug: %s\n", msg);
+	
+	return;
+}
+
+/* Add the current username (from the Login window) to the list of user accounts */
+
+void
+add_account_to_list (DrivelClient *dc)
+{
+	DrivelUser *du;
+	GSList *du_existing, *item;
+
+	debug ("add_account_to_list()");
+	
+	/* copy the current user into du */
+	du = drivel_user_new ();
+	du->username = g_strdup (dc->user->username);
+	du->password = g_strdup (dc->user->password);
+	du->api = dc->user->api;
+	du->server = g_strdup (dc->user->server);
+	du->save_password = dc->user->save_password;
+	du->autologin = dc->user->autologin;
+	du->lastuser = TRUE;
+	
+	/* update the user list with the most recent login information */
+	du_existing = find_in_user_list (dc->user_list, du);
+	if (du_existing)
+	{
+		drivel_user_free (du_existing->data);
+		dc->user_list = g_slist_delete_link (dc->user_list, du_existing);
+	}
+	/* if the user doesn't want to save her password, remove it before saving
+	   the list to disk */
+	if (!du->save_password)
+	{
+		g_free (du->password);
+		du->password = g_strdup ("");
+	}
+	/* logging in as one user turns off autologin on any other users */
+	for (item = dc->user_list; item; item = item->next)
+	{
+		DrivelUser *du;
+		du = item->data;
+		du->autologin = FALSE;
+		du->lastuser = FALSE;
+	}
+	dc->user_list = g_slist_prepend (dc->user_list, du);
+	save_user_list (dc->user_list, dc->config_directory);
+	user_list_changed (dc);
+	drivel_gconf_data_fill (dc->gconf, du->username, du->api, dc->client,
+			&dc->id, dc);
+	
+	return;
+}
+
+/* Sort the journal list alphabetically with single-user journals listed first */
+gint
+sort_journals (DrivelJournal *a, DrivelJournal *b)
+{
+	gint retval;
+	
+	retval = a->type - b->type;
+	if (!retval)
+		retval = strcmp (a->name, b->name);
+
+	return retval;
+}
+
+/* Clear any elements in the recent_entries array */
+void
+clear_recent_entries (GArray *recent)
+{
+	while (recent->len)
+	{
+		DrivelJournalEntry *entry;
+		
+		entry = g_array_index (recent, DrivelJournalEntry*, 0);
+		g_array_remove_index (recent, 0);
+		journal_entry_free (entry);
+	}
+	
+	return;
+}
+
+void
+fill_time (DrivelClient *dc)
 {
     time_t exact_time;
     struct tm *local_time;
     
     /* we need the year, month, day, hour, and minute in order to post */
-    exact_time = time(NULL);
-    local_time = localtime(&exact_time);
+    exact_time = time (NULL);
+    local_time = localtime (&exact_time);
     
     dc->time.year = (local_time->tm_year + 1900);
     dc->time.month = (local_time->tm_mon + 1);
@@ -140,7 +245,7 @@ validate_username (const gchar *username)
 	{
 		c = g_utf8_get_char_validated (p, -1);
 
-		if (!g_unichar_isalnum (c) && c != '_')
+		if (!g_unichar_isalnum (c) && c != '_' && c!= '-')
 		{
 			retval = FALSE;
 			break;
@@ -193,15 +298,6 @@ get_default_text (GConfClient *client, const gchar *key, const gchar *standard_t
 	return string;
 }
 
-void
-list_free_item (gpointer data, gpointer user_data)
-{
-	g_free (data);
-	data = NULL;
-	
-	return;
-}
-
 void
 menu_list_free_item (gpointer data, gpointer user_data)
 {
@@ -235,6 +331,22 @@ compare_usernames (gconstpointer a, gconstpointer b)
 	return (strncmp (friend->username, username, strlen (friend->username)));
 }
 
+/* 'security_store' must already be a valid GtkListStore */
+void
+fill_security_menu (DrivelClient *dc, GtkListStore *security_store)
+{
+	gtk_list_store_clear (security_store);
+	
+	store_security_append (security_store, "public.png", _("Public"), 
+			-10, -10, TRUE);
+	store_security_append (security_store, "protected.png", _("Friends Only"), 
+			0, -5, TRUE);
+	store_security_append (security_store, "private.png", _("Private"), 
+			-5, 256, TRUE);
+	
+	return;
+}
+
 /* 'picture_store' must already be a valid GtkListStore */
 void
 fill_picture_menu (DrivelClient *dc, GtkListStore *picture_store)
@@ -244,9 +356,14 @@ fill_picture_menu (DrivelClient *dc, GtkListStore *picture_store)
 	const gchar *text;
 	GdkPixbuf *pixbuf, *pixbuf_small;
 	GtkTreeIter iter;
-
+	
+	debug ("fill_picture_menu()");
+	
 	gtk_list_store_clear (picture_store);
 	
+	if (!dc->picture_keywords || !dc->picture_filenames)
+		return;
+	
 	for (i = 0; i < dc->pictures + 1; i++)
 	{
 		gchar *pickf;
@@ -267,7 +384,30 @@ fill_picture_menu (DrivelClient *dc, GtkListStore *picture_store)
 				g_warning ("Couldn't load %s: %s\n", pickf, error->message);
 			if (pixbuf && !error)
 			{
-				pixbuf_small = gdk_pixbuf_scale_simple (pixbuf, 20, 20, GDK_INTERP_BILINEAR);
+				gdouble ratio;
+				gint width, height;
+				
+				/* scale the icon while preserving it's dimension ratio */
+				width = gdk_pixbuf_get_width (pixbuf);
+				height = gdk_pixbuf_get_height (pixbuf);
+				if (width > ICON_SIZE || height > ICON_SIZE)
+				{
+					if (width < height)
+					{
+						ratio = (gdouble)width / height;
+						width = (gint)ceil (ratio * ICON_SIZE);
+						height = ICON_SIZE;
+					}
+					else
+					{
+						ratio = (gdouble)height / width;
+						width = ICON_SIZE;
+						height = (gint)ceil (ratio * ICON_SIZE);
+					}
+				}
+				
+				pixbuf_small = gdk_pixbuf_scale_simple (pixbuf, width, height, 
+						GDK_INTERP_BILINEAR);
 				g_object_unref (G_OBJECT (pixbuf));
 			}
 			else
@@ -326,25 +466,15 @@ gconf_data_free (DrivelGConfData *data, GConfClient *client, DrivelIDs *id)
 		remove_gconf_notifies (client, id);
 
 	g_free (data->user);
-	g_free (data->password);
-	g_free (data->save_password);
-	g_free (data->autologin);
 	
-	g_free (data->default_server_list);
-	g_free (data->default_server);
 	g_free (data->default_mood);
 	g_free (data->default_music);
 	g_free (data->default_picture);
 	g_free (data->default_security);
+	g_free (data->default_security_mask);
 	g_free (data->default_comment);
 	g_free (data->default_autoformat);
 	
-	g_free (data->proxy);
-	g_free (data->proxy_url);
-	g_free (data->proxy_port);
-	g_free (data->proxy_user);
-	g_free (data->proxy_pass);
-	
 	g_free (data->tray);
 	
 	g_free (data->entry_x);
@@ -357,41 +487,39 @@ gconf_data_free (DrivelGConfData *data, GConfClient *client, DrivelIDs *id)
 	g_free (data->min_post);
 	g_free (data->highlight_syntax);
 	g_free (data->spellcheck);
+	g_free (data->expander_open);
+	g_free (data->spell_language);
+	g_free (data->last_journal);
+	g_free (data->technorati);
 	
 	return;
 }
 
 void
-drivel_gconf_data_fill (DrivelGConfData *data, const gchar *username, GConfClient *client, DrivelIDs *id, DrivelClient *dc)
+drivel_gconf_data_fill (DrivelGConfData *data, const gchar *username, 
+		DrivelBlogAPI api, GConfClient *client, DrivelIDs *id, DrivelClient *dc)
 {
 	gchar *base;
+	gchar *uid;
 	
 	g_return_if_fail (username);
 	
 	if (data->user)
 		gconf_data_free (data, client, id);
 	
-	base = g_strdup_printf ("/apps/drivel/users/%s", username);
+	/* the uid is the username and api concated together */
+	uid = g_strdup_printf ("%s_%d", username, api);
+	
+	base = g_strdup_printf ("/apps/drivel/users/%s", uid);
 	data->user = base;
-	data->password = g_strdup_printf ("%s/password", base);
-	data->save_password = g_strdup_printf ("%s/save_password", base);
-	data->autologin = g_strdup_printf ("%s/autologin", base);
 	
-	data->default_server_list = g_strdup_printf ("%s/default_server_list", base);
-	data->default_server = g_strdup_printf ("%s/default_server", base);
 	data->default_mood = g_strdup_printf ("%s/default_mood", base);
 	data->default_music = g_strdup_printf ("%s/default_music", base);
 	data->default_picture = g_strdup_printf ("%s/default_picture", base);
-	data->default_security = g_strdup_printf ("%s/default_security", base);
+	data->default_security = g_strdup_printf ("%s/default_security_name", base);
+	data->default_security_mask = g_strdup_printf ("%s/default_security_mask", base);
 	data->default_comment = g_strdup_printf ("%s/default_comment", base);
 	data->default_autoformat = g_strdup_printf ("%s/default_autoformat", base);
-	data->expander_open = g_strdup_printf ("%s/expander_open", base);
-
-	data->proxy = g_strdup ("/system/http_proxy/use_http_proxy");
-	data->proxy_url = g_strdup ("/system/http_proxy/host");
-	data->proxy_port = g_strdup ("/system/http_proxy/port");
-	data->proxy_user = g_strdup ("/system/http_proxy/authentication_user");
-	data->proxy_pass = g_strdup ("/system/http_proxy/authentication_password");
 	
 	data->tray = g_strdup_printf ("%s/tray", base);
 	
@@ -405,6 +533,11 @@ drivel_gconf_data_fill (DrivelGConfData *data, const gchar *username, GConfClien
 	data->min_post = g_strdup_printf ("%s/min_post", base);
 	data->highlight_syntax = g_strdup_printf ("%s/highlight_syntax", base);
 	data->spellcheck = g_strdup_printf ("%s/spellcheck", base);
+	data->spell_language = g_strdup_printf ("%s/spell_language", base);
+	data->last_journal = g_strdup_printf ("%s/last_journal", base);
+	data->technorati = g_strdup_printf ("%s/technorati", base);
+	
+	data->expander_open = g_strdup_printf ("%s/expander_open", base);
 	
 	gconf_client_add_dir (client, base, GCONF_CLIENT_PRELOAD_NONE, NULL);
 
@@ -419,6 +552,7 @@ drivel_register_icons (void)
 	GtkIconFactory *factory;
 	GtkIconSet *icon;
 	GdkPixbuf *pixbuf;
+	gchar *path;
 	GtkStockItem items[] =
 	{
 		{
@@ -485,15 +619,15 @@ drivel_register_icons (void)
 			NULL
 		},
 		{
-			"drivel-dont-save",
-			_("_Don't Save"),
+			"drivel-save-draft-btn",
+			_("Save _Draft"),
 			0,
 			0,
 			NULL
 		},
 		{
-			"drivel-sign-up",
-			_("Sign _Up..."),
+			"drivel-dont-save",
+			_("_Don't Save"),
 			0,
 			0,
 			NULL
@@ -512,33 +646,26 @@ drivel_register_icons (void)
 			0,
 			NULL
 		},
-		{
-			"drivel-what-is-lj",
-			_("What's a LiveJournal?"),
-			0,
-			0,
-			NULL
-		}
 	};
 	
 	factory = gtk_icon_factory_new ();
 	
 	/* create our own icons */
-	pixbuf = gdk_pixbuf_new_from_file (
-		DATADIR G_DIR_SEPARATOR_S "pixmaps" G_DIR_SEPARATOR_S "livejournal.png",
-		NULL);
+	path = g_build_filename (DATADIR, "pixmaps", "drivel", "drivel-24.png", NULL);
+	pixbuf = gdk_pixbuf_new_from_file (path, NULL);
+	g_free (path);
 	icon = gtk_icon_set_new_from_pixbuf (pixbuf);
 	gtk_icon_factory_add (factory, "drivel-post", icon);
 	
-	pixbuf = gdk_pixbuf_new_from_file (
-		DATADIR G_DIR_SEPARATOR_S "pixmaps" G_DIR_SEPARATOR_S "drivel" 
-		G_DIR_SEPARATOR_S "insert_image.png", NULL);
+	path = g_build_filename (DATADIR, "pixmaps", "drivel", "insert_image.png", NULL);
+	pixbuf = gdk_pixbuf_new_from_file (path, NULL);
+	g_free (path);
 	icon = gtk_icon_set_new_from_pixbuf (pixbuf);
 	gtk_icon_factory_add (factory, "drivel-insert-image", icon);
 	
-	pixbuf = gdk_pixbuf_new_from_file (
-		DATADIR G_DIR_SEPARATOR_S "pixmaps" G_DIR_SEPARATOR_S "drivel" 
-		G_DIR_SEPARATOR_S "insert_object.png", NULL);
+	path = g_build_filename (DATADIR, "pixmaps", "drivel", "insert_object.png", NULL);
+	pixbuf = gdk_pixbuf_new_from_file (path, NULL);
+	g_free (path);
 	icon = gtk_icon_set_new_from_pixbuf (pixbuf);
 	gtk_icon_factory_add (factory, "drivel-insert-link", icon);
 	
@@ -564,15 +691,15 @@ drivel_register_icons (void)
 	icon = gtk_icon_factory_lookup_default (GTK_STOCK_SAVE);
 	gtk_icon_factory_add (factory, "drivel-save-draft", icon);
 	
+	icon = gtk_icon_factory_lookup_default (GTK_STOCK_SAVE);
+	gtk_icon_factory_add (factory, "drivel-save-draft-btn", icon);
+	
 	icon = gtk_icon_factory_lookup_default (GTK_STOCK_NO);
 	gtk_icon_factory_add (factory, "drivel-dont-save", icon);
 	
 	icon = gtk_icon_factory_lookup_default (GTK_STOCK_SAVE);
 	gtk_icon_factory_add (factory, "drivel-update", icon);
 	
-	icon = gtk_icon_factory_lookup_default (GTK_STOCK_HELP);
-	gtk_icon_factory_add (factory, "drivel-what-is-lj", icon);
-	
 	gtk_icon_factory_add_default (factory);
 	gtk_stock_add (items, G_N_ELEMENTS (items));
 	
@@ -628,6 +755,54 @@ init_config_directory (void)
 	return directory;
 }
 
+static gint
+security_cmp (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer data)
+{
+	gint group1, group2;
+	
+	gtk_tree_model_get (model, a, STORE_SECURITY_ORDER, &group1, -1);
+	gtk_tree_model_get (model, b, STORE_SECURITY_ORDER, &group2, -1);
+	
+	return group1 - group2;
+}
+
+static gint
+category_cmp (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer data)
+{
+	gchar *cat1, *cat2;
+	gint retval;
+	
+	gtk_tree_model_get (model, a, STORE_CATEGORY_NAME, &cat1, -1);
+	gtk_tree_model_get (model, b, STORE_CATEGORY_NAME, &cat2, -1);
+	
+	if (!strcmp (_("None"), cat1))
+		retval = -1;
+	else if (!strcmp (_("None"), cat2))
+		retval = 1;
+	else
+		retval = strcmp (cat1, cat2);
+	
+	return retval;
+}
+
+static gboolean
+is_category_visible (GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
+{
+	DrivelClient *dc = (DrivelClient *)data;
+	gchar *category_name = NULL, *journal_name = NULL;
+	gboolean retval = FALSE;
+	
+	if (dc->active_journal)
+		journal_name = dc->active_journal->name;
+	
+	gtk_tree_model_get (model, iter, STORE_CATEGORY_BLOG, &category_name, -1);
+	if (category_name && journal_name && 
+		(!strcmp (category_name, journal_name) || !strcmp (category_name, "")))
+		retval = TRUE;
+	
+	return retval;
+}
+
 static void
 drivel_init (DrivelClient *dc)
 {
@@ -642,7 +817,6 @@ drivel_init (DrivelClient *dc)
 	dc->net = g_new0 (DrivelNet, 1);
 	/* FIXME: this needs to be done a per-account basis */
 	dc->net->api = BLOG_API_LJ;
-	dc->net->server = NULL;
 	dc->net->curl_share = curl_share_init ();
 	if (!dc->net->curl_share)
 		g_warning ("Could not access the curl_share interface.");
@@ -653,35 +827,65 @@ drivel_init (DrivelClient *dc)
 	dc->net->msg_queue = msg_queue_setup ();
 	dc->net->cancel = gnome_vfs_cancellation_new ();
 	dc->gconf->user = NULL;
-	dc->username = NULL;
-	dc->password = NULL;
+	dc->user = drivel_user_new ();
 	dc->picture_keywords = NULL;
 	dc->picture_filenames = NULL;
-	dc->open_dialog = NULL;
-	dc->save_dialog = NULL;
-	dc->network_progress = NULL;
-	dc->response = 0;
-	dc->message_dialog = NULL;
 	dc->use_fast_servers = TRUE;
-	dc->proxy_user = NULL;
-	dc->proxy_pass = NULL;
-	dc->proxy_url = NULL;
-	dc->proxy_proper_format = NULL;
+	dc->proxy = gconf_client_get_bool (dc->client, 
+			"/system/http_proxy/use_http_proxy", NULL);
+	dc->proxy_auth = gconf_client_get_bool (dc->client, 
+			"/system/http_proxy/use_authentication", NULL);
+	dc->proxy_url = gconf_client_get_string (dc->client, 
+			"/system/http_proxy/host", NULL);
+	dc->proxy_port = gconf_client_get_int (dc->client, 
+			"/system/http_proxy/port", NULL);
+	dc->proxy_user = gconf_client_get_string (dc->client,
+			"/system/http_proxy/authentication_user", NULL);
+	dc->proxy_pass = gconf_client_get_string (dc->client,
+			"/system/http_proxy/authentication_password", NULL);
+	dc->journal_entry = NULL;
 	dc->journals = 0;
 	dc->journal_list = NULL;
 	dc->active_journal = NULL;
 	dc->menu_list = NULL;
 	dc->friends_list = NULL;
-	dc->login_send_data = NULL;
-	dc->invalid_input = FALSE;
 	dc->edit_entry = FALSE;
 	dc->window_list = NULL;
 	dc->time_since_checkfriends = 0;
 	dc->checking_friends = FALSE;
 	dc->friends_update = FALSE;
 	dc->lastupdate = g_strdup ("");
-	dc->picture_store = NULL;
+	dc->picture_store = NULL; /* FIXME: allocate this here */
+	dc->security_store = gtk_list_store_new (STORE_SECURITY_N,
+			GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT, 
+			G_TYPE_BOOLEAN);
+	gtk_tree_sortable_set_sort_func (
+			GTK_TREE_SORTABLE (dc->security_store), STORE_SECURITY_ORDER, 
+			security_cmp, NULL, NULL);
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (dc->security_store),
+			STORE_SECURITY_ORDER, GTK_SORT_ASCENDING);
+	dc->security_store_filtered = gtk_list_store_new (STORE_SECURITY_N,
+			GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT, 
+			G_TYPE_BOOLEAN);
+	gtk_tree_sortable_set_sort_func (
+			GTK_TREE_SORTABLE (dc->security_store), STORE_SECURITY_ORDER, 
+			security_cmp, NULL, NULL);
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (dc->security_store),
+			STORE_SECURITY_ORDER, GTK_SORT_ASCENDING);
+	dc->category_store = gtk_list_store_new (STORE_CATEGORY_N, 
+			G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
+	gtk_tree_sortable_set_sort_func (
+			GTK_TREE_SORTABLE (dc->category_store), STORE_CATEGORY_NAME, 
+			category_cmp, NULL, NULL);
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (dc->category_store),
+			STORE_CATEGORY_NAME, GTK_SORT_ASCENDING);
+	dc->category_store_filtered = gtk_tree_model_filter_new (
+			GTK_TREE_MODEL (dc->category_store), NULL);
+	gtk_tree_model_filter_set_visible_func (
+			GTK_TREE_MODEL_FILTER (dc->category_store_filtered),
+			is_category_visible, dc, NULL);
 	dc->draft_filename = NULL;
+	dc->recent_entries = g_array_new (FALSE, TRUE, sizeof (DrivelJournalEntry *));
 	
 	drivel_fill_journal_null (dc);
 	
@@ -703,10 +907,16 @@ main (gint argc, gchar **argv)
 	DrivelClient *dc;
 	GnomeProgram *program;
 	GError *error;
+	struct poptOption options[] = 
+	{
+		{ "verbose", 'v', POPT_ARG_NONE, &verbose, 0, "Display debugging output", "" },
+		{ NULL }
+	};
 	
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
+	xmlInitParser ();
 	
 	error = NULL;
 	if (gconf_init (argc, argv, &error) == FALSE)
@@ -722,28 +932,29 @@ main (gint argc, gchar **argv)
 	
 	program = gnome_program_init (PACKAGE, VERSION, 
 			LIBGNOMEUI_MODULE, argc, argv,
+			GNOME_PARAM_POPT_TABLE, options,
 			GNOME_PARAM_HUMAN_READABLE_NAME, _("Drivel Journal Editor"),
 			GNOME_PARAM_APP_DATADIR, DATADIR,
 			NULL);
 	
-	curl_global_init (CURL_GLOBAL_NOTHING);
+	curl_global_init (CURL_GLOBAL_SSL);
 
 	net_mutex = g_mutex_new ();
-	
+
+	/* Build and initialize the DrivelClient */
 	dc = g_new0 (DrivelClient, 1);
-
 	drivel_init (dc);
-
-	drivel_register_icons ();
-
-	net_start_thread (dc);
 	
+	/* Check if the user passed a filename as an argument */
+	if ((argc > 1) && (argv[argc - 1][0] != '-'))
+		dc->draft_filename = g_strdup (argv[argc - 1]);
+	
+	drivel_register_icons ();
+	net_start_thread (dc);
 	login_window_build (dc);
 	
 	gdk_threads_enter ();
-	
 	gtk_main ();
-	
 	gdk_threads_leave ();
 	
 	return 0;
